<?php
///////////////////////////////////////////////////////////////////////////

$err = fopen("php://stderr", "w");
function hd_print($str) {
    global $err;
    fwrite($err, "$str\n");
}

function error($msg) {
    return array("error" => array("message" => $msg));
}

function finish($res) {
    $out = json_encode($res);
    hd_print("Error: $out");
    echo("$out\n");
    exit(0);
}

set_error_handler('hd_error_handler');
register_shutdown_function('hd_shutdown_handler');

///////////////////////////////////////////////////////////////////////////

if (count($argv) != 2)
    finish(error("bad number of arguments"));

$input_str = file_get_contents("php://stdin");
$input_obj = json_decode($input_str);
if (!isset($input_obj))
    finish(error("failed to decode input json"));

$my_dir = dirname($argv[0]);
$cmd = $argv[1];

/*
if ($cmd == 'get_episodes')
    $res = get_episodes($input_obj, $my_dir);
else if ($cmd == 'do_file_search')
    $res = do_file_search($input_obj, $my_dir);
else if ($cmd == 'get_stream_url')
    $res = get_stream_url($input_obj);
else
*/
    finish(error("unknown command: $cmd"));

finish($res);

///////////////////////////////////////////////////////////////////////////

function hd_error_handler($error_type, $message, $file, $line)
{
    static $map = array
    (
        E_ERROR             => 'E_ERROR',
        E_WARNING           => 'E_WARNING',
        E_PARSE             => 'E_PARSE',
        E_NOTICE            => 'E_NOTICE',
        E_CORE_ERROR        => 'E_CORE_ERROR',
        E_CORE_WARNING      => 'E_CORE_WARNING',
        E_COMPILE_ERROR     => 'E_COMPILE_ERROR',
        E_COMPILE_WARNING   => 'E_COMPILE_WARNING',
        E_USER_ERROR        => 'E_USER_ERROR',
        E_USER_WARNING      => 'E_USER_WARNING',
        E_USER_NOTICE       => 'E_USER_NOTICE',
        E_STRICT            => 'E_STRICT',
        E_RECOVERABLE_ERROR => 'E_RECOVERABLE_ERROR',
        E_DEPRECATED        => 'E_DEPRECATED',
        E_USER_DEPRECATED   => 'E_USER_DEPRECATED'
    );

    if (isset($map[$error_type]))
    {
        if ($error_type == E_USER_ERROR)
        {
            $etype = $map[$error_type];
            finish(error("PHP error [$file:$line] [$etype] $message"));
        }
        hd_print("PHP error [$file:$line] [$etype] $message");
        return true;
    }

    return false;
}

function hd_shutdown_handler() {
  $err = error_get_last();
  hd_error_handler($err['type'], $err['message'], $err['file'], $err['line']);
}

function hd_error_silencer($severity, $msg, $filename, $linenum)
{ }

function hd_silence_warnings() {
    error_reporting(0);
    set_error_handler("hd_error_silencer");
}

function hd_restore_warnings() {
    restore_error_handler();
    error_reporting(E_STRICT | E_ALL);
}

///////////////////////////////////////////////////////////////////////////

class HD
{
    const INT_MIN = -2147483648;
    const INT_MAX = 2147483647;

    public static $DEFAULT_CURL_TIMEOUT = 40;
    public static $DEFAULT_CURL_CONNECT_TIMEOUT = 40;

    public static function http_get_document($url, $opts = null)
    {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER,     0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,     0);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT,    40);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION,    true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,    true);
        curl_setopt($ch, CURLOPT_TIMEOUT,           40);
        curl_setopt($ch, CURLOPT_USERAGENT,         'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.122 Safari/537.36');
		curl_setopt($ch, CURLOPT_ENCODING,          'gzip,deflate');
        curl_setopt($ch, CURLOPT_URL,               $url);

        if (isset($opts))
        {
            foreach ($opts as $k => $v)
                curl_setopt($ch, $k, $v);
        }

        hd_print("HTTP fetching '$url'...");

        if (isset($opts) && isset($opts[CURLOPT_POSTFIELDS]))
            hd_print("HTTP POST fields '" . $opts[CURLOPT_POSTFIELDS] . "'");

        $content = curl_exec($ch);
        $curl_errno = curl_errno($ch);
        $curl_error = curl_error($ch);

        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if($content === false)
        {
            $err_msg =
                "CURL errno: $curl_errno ($curl_error); ".
                "HTTP error: $http_code; ";
            hd_print($err_msg);
        }
        else if ($http_code != 200)
        {
            $err_msg = "HTTP request failed ($http_code)";
            hd_print($err_msg);
        }
        else
            hd_print("HTTP OK ($http_code)");

        curl_close($ch);

        return $content;
    }
}

