<?php

require_once 'lib/properties.php';

class Config extends Properties
{
    const SWITCH_LONG_OPEN = 10;
    const SWITCH_LONG_START = 15;
    const SWITCH_LONG_SEEK = 30;
    const SWITCH_LONG_START_BUFFERING = 30;
    const SWITCH_LONG_SEEK_BUFFERING = 40;
    const SWITCH_LONG_BUFFERING = 10;
    const SWITCH_FREQUENT_BUFFERING_PERIOD = 60;
    const SWITCH_FREQUENT_BUFFERING_COUNT = 6;
    const AUTOSWITCH_ENABLED = 1;

    public static $temp_force_orig_torlook_addr = -1;

    public function __construct()
    {
        parent::__construct('/config/omov_config.txt', 1);
        $this->validate();
    }

    private function get_autoswitch_enabled()
    {
        return is_file("/config/online_movies_full.flag") ?
            self::AUTOSWITCH_ENABLED : 0;
    }

    protected function validate()
    {
        $this->set_default('debug_forced_switch_delay', 0);
        $this->set_default('autoswitch_enabled', $this->get_autoswitch_enabled());
        $this->set_default('switch_long_open', self::SWITCH_LONG_OPEN);
        $this->set_default('switch_long_start', self::SWITCH_LONG_START);
        $this->set_default('switch_long_seek', self::SWITCH_LONG_SEEK);
        $this->set_default('switch_long_start_buffering', self::SWITCH_LONG_START_BUFFERING);
        $this->set_default('switch_long_seek_buffering', self::SWITCH_LONG_SEEK_BUFFERING);
        $this->set_default('switch_long_buffering', self::SWITCH_LONG_BUFFERING);
        $this->set_default('switch_frequent_buffering_period', self::SWITCH_FREQUENT_BUFFERING_PERIOD);
        $this->set_default('switch_frequent_buffering_count', self::SWITCH_FREQUENT_BUFFERING_COUNT);
        $this->set_default('use_orig_torlook_addr', 0);

        if ($this->switch_frequent_buffering_period < 10)
            $this->switch_frequent_buffering_period = 10;
        if ($this->switch_frequent_buffering_count < 2)
            $this->switch_frequent_buffering_count = 2;

        if ($this->use_orig_torlook_addr()) {
            TorLook_Class::setDomainName('https://torlook.info');
            TorLook_Class::$customResolveOpts = '';
        } else {
            // EEE
            //TorLook_Class::setDomainName('https://gw2.torlook.info');
            TorLook_Class::setDomainName('https://gw3.torlook.info');
            TorLook_Class::$customResolveOpts = '';
            #TorLook_Class::$customResolveOpts = '--resolve gw2.torlook.info:443:185.43.221.227';
        }
    }

    public function debug_forced_switch_delay() {
        return (int) $this->debug_forced_switch_delay;
    }

    public function autoswitch_enabled() {
        return (int) $this->autoswitch_enabled;
    }

    public function switch_long_open() {
        return (int) $this->switch_long_open;
    }

    public function switch_long_start() {
        return (int) $this->switch_long_start;
    }

    public function switch_long_seek() {
        return (int) $this->switch_long_seek;
    }

    public function switch_long_start_buffering() {
        return (int) $this->switch_long_start_buffering;
    }

    public function switch_long_seek_buffering() {
        return (int) $this->switch_long_seek_buffering;
    }

    public function switch_long_buffering() {
        return (int) $this->switch_long_buffering;
    }

    public function switch_frequent_buffering_period() {
        return (int) $this->switch_frequent_buffering_period;
    }

    public function switch_frequent_buffering_count() {
        return (int) $this->switch_frequent_buffering_count;
    }

    public function use_orig_torlook_addr() {
        if (self::$temp_force_orig_torlook_addr === 1)
            return 1;
        return (int) $this->use_orig_torlook_addr;
    }

    public function reload() {
        parent::reload();
    }

}

?>
