<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/text_utils.php';
require_once 'lib/media_url.php';
require_once 'lib/user_input_handler_registry.php';
require_once 'lib/action_factory.php';
require_once 'lib/control_factory.php';

class DefaultDunePlugin implements DunePlugin
{
    private $screens;

    private $plugin_cookies = null;
    private $opexec_id = -1;

    public function set_plugin_cookies($plugin_cookies)
    {
        $this->plugin_cookies = $plugin_cookies;
    }

    public function &get_plugin_cookies()
    {
        return $this->plugin_cookies;
    }

    public function set_internet_status($internet_status)
    {
        $this->internet_status = $internet_status;
    }

    public function get_internet_status()
    {
        return $this->internet_status;
    }

    public function set_opexec_id($opexec_id)
    {
        $this->opexec_id = $opexec_id;
    }

    public function get_opexec_id()
    {
        return $this->opexec_id;
    }

    ///////////////////////////////////////////////////////////////////////

    protected function __construct()
    {
        $this->screens = array();
    }

    ///////////////////////////////////////////////////////////////////////
    //
    // Screen support.
    //
    ///////////////////////////////////////////////////////////////////////

    protected function add_screen($scr)
    {
        if (isset($this->screens[$scr->get_id()]))
        {
            hd_print("Error: screen (id: " . $scr->get_id() . ") already registered.");
            throw new Exception('Screen already registered');
        }

        $this->screens[$scr->get_id()] = $scr;
    }

    ///////////////////////////////////////////////////////////////////////

    protected function get_screen_by_id($screen_id)
    {
        if (isset($this->screens[$screen_id]))
            return $this->screens[$screen_id];

        hd_print("Error: no screen with id '$screen_id' found.");
        throw new Exception('Screen not found');
    }

    ///////////////////////////////////////////////////////////////////////////

    protected function get_screen_by_url($media_url)
    {
        $screen_id = 
            isset($media_url->screen_id) ?
            $media_url->screen_id :
            $media_url->get_raw_string();

        return $this->get_screen_by_id($screen_id);
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    // Folder support.
    //
    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view($media_url_str, &$plugin_cookies)
    {
        $media_url = MediaURL::decode($media_url_str);
        $screen = $this->get_screen_by_url($media_url);

        return $screen->get_folder_view($media_url, $plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////////

    public function get_next_folder_view($media_url_str, &$plugin_cookies)
    {
        $media_url = MediaURL::decode($media_url_str);

        return
            $this->
                get_screen_by_url($media_url)->
                    get_next_folder_view($media_url, $plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////////

    public function get_regular_folder_items($media_url_str, $from_ndx,
        &$plugin_cookies)
    {
        $media_url = MediaURL::decode($media_url_str);

        return $this->get_screen_by_url($media_url)->
            get_folder_range($media_url, $from_ndx, $plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////
    //
    // IPTV channels support (TV support).
    //
    ///////////////////////////////////////////////////////////////////////

    public function get_tv_info($media_url_str, &$plugin_cookies)
    {
        throw new Exception('TV is not supported');
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_tv_stream_url($playback_url, &$plugin_cookies)
    { 
        throw new Exception('TV is not supported');
    }
    
    ///////////////////////////////////////////////////////////////////////

    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
    {
        throw new Exception('TV is not supported');
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_day_epg($channel_id, $day_start_ts, &$plugin_cookies)
    {
        throw new Exception('TV is not supported');
    }

    ///////////////////////////////////////////////////////////////////////

    public function change_tv_favorites($fav_op_type, $channel_id, &$plugin_cookies)
    {
        throw new Exception('TV is not supported');
    }

    ///////////////////////////////////////////////////////////////////////
    //
    // VOD support.
    //
    ///////////////////////////////////////////////////////////////////////

    public function get_vod_info($media_url_str, &$plugin_cookies)
    {
        throw new Exception('VOD is not supported');
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_vod_stream_url($playback_url, &$plugin_cookies)
    {
        $pb = MainHelper::$playback;
        return $pb->get_vod_stream_url($playback_url);
    }

    ///////////////////////////////////////////////////////////////////////
    //
    // Misc.
    //
    ///////////////////////////////////////////////////////////////////////

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        // XXX DEBUG
/*
        $handler_id = HD::get($user_input, 'handler_id', '?');
        $control_id = HD::get($user_input, 'control_id',
            HD::get($user_input, 'entry_id', '?'));
        hd_print("XXX DEBUG handle_user_input (req $this->opexec_id): $handler_id->$control_id");
*/

        $res = UserInputHandlerRegistry::get_instance()->handle_user_input(
            $user_input, $plugin_cookies);

/*
        $astr = HD::get($res, 'handler_string_id', 'none');
        hd_print("XXX DEBUG handle_user_input returned action: $astr");
*/

        return $res;
    }
}

///////////////////////////////////////////////////////////////////////////
?>
