<?php
///////////////////////////////////////////////////////////////////////////

class Properties
{
    private $path;
    private $data;
    public $verbose = 1;

    public function __construct($path, $verbose=1)
    {
        $this->path = $path;
        $this->data = array();
        $this->verbose = $verbose;

        $this->read_properties_file($this->path);
    }

    public function reload()
    {
        $this->data = array();
        $this->read_properties_file($this->path);
        $this->validate();
    }

    private function read_properties_file($path)
    {
        return self::do_read_properties_file(
            $this->data, $path, $this->verbose);
    }

    public static function do_parse_lines(&$data, $lines)
    {
        if (!$lines)
            return true;

        for ($i = 0; $i < count($lines); ++$i)
        {
            $line = trim($lines[$i]);
            if (strlen($line) == 0 || $line[0] == '#')
                continue;

            if (preg_match('/^([a-zA-Z_][A-Za-z0-9_\-:\.]*) *= *(.*)$/',
                    $line, $matches) != 1)
            {
                hd_print(
                    "Warning: line " . ($i + 1) . ": unknown format. " .
                    "Data: '" . $lines[$i] . "'.");
                continue;
            }

            $data[$matches[1]] = $matches[2];
        }

        return true;
    }

    public static function do_parse_string(&$data, $text)
    {
        if (!$text)
            return true;

        $lines = explode("\n", $text);
        return self::do_parse_lines($data, $lines);
    }

    public static function do_read_properties_file(&$data,
        $path, $verbose)
    {
        hd_silence_warnings();
        $lines = file($path, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        hd_restore_warnings();

        if ($lines === false)
        {
            if ($verbose)
                hd_print("Properties file '$path' does not exist.");
            return false;
        }

        if ($verbose)
            hd_print("Reading properties from '$path'...");

        return self::do_parse_lines($data, $lines);
    }

    public function __get($key)
    { return isset($this->data[$key]) ? $this->data[$key] : null; }

    public function __isset($key)
    { return isset($this->data[$key]); }

    public function set($key, $value) {
        $this->data[$key] = $value;
    }

    public function set_default($key, $value)
    {
        if (!isset($this->data[$key]))
        {
            if ($this->verbose >= 2)
                hd_print("Warning: no value for key '$key'. Using default: '$value'");
            $this->data[$key] = $value;
        }
    }

    public function log_option($key)
    {
        if (isset($this->data[$key]))
            hd_print("$key: '" . $this->data[$key] . "'");
        else
            hd_print("$key is not set.");
    }

    protected function validate() {
        // To override.
    }
}

///////////////////////////////////////////////////////////////////////////
?>
