<?php
///////////////////////////////////////////////////////////////////////////

class HD
{
    const INT_MIN = -2147483648;
    const INT_MAX = 2147483647;

    const DUNE_AUTH_PATH = "/tmp/dune_auth.txt";

    /////////////////////////////////////////////////////////////

    public static function is_apk() {
        return (int) getenv('HD_APK');
    }

    /////////////////////////////////////////////////////////////

    public static function get($map, $key, $def=null)
    {
        if (is_object($map))
            return isset($map->$key) ? $map->$key : $def;
        if (is_array($map))
            return isset($map[$key]) ? $map[$key] : $def;
        return $def;
    }

    public static function arrget($arr, $key, $def=null)
    {
        return isset($arr[$key]) ? $arr[$key] : $def;
    }

    /////////////////////////////////////////////////////////////

    public static function is_map($a)
    {
        return is_array($a) &&
            array_diff_key($a, array_keys(array_keys($a)));
    }

    ///////////////////////////////////////////////////////////////////////

    public static function has_attribute($obj, $n)
    {
        $arr = (array) $obj;
        return isset($arr[$n]);
    }
    ///////////////////////////////////////////////////////////////////////

    public static function get_map_element($map, $key)
    {
        return isset($map[$key]) ? $map[$key] : null;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function starts_with($str, $pattern)
    {
        return strpos($str, $pattern) === 0;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function format_timestamp($ts, $fmt = null)
    {
        // NOTE: for some reason, explicit timezone is required for PHP
        // on Dune (no builtin timezone info?).

        if (is_null($fmt))
            $fmt = 'Y:m:d H:i:s';

        $dt = new DateTime('@' . $ts);
        return $dt->format($fmt);
    }

    ///////////////////////////////////////////////////////////////////////

	public static function readableBytes($bytes, $bytesEnd=null, $time=null) {
		if ($bytesEnd== true)
			$bytes = ($bytes - $bytesEnd) / $time;
		if (intval($bytes) == 0)
			return '0 B';
		$bytes = floatval($bytes);
		$i = floor(log($bytes) / log(1024));
		$sizes = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
		return sprintf('%.02F', $bytes / pow(1024, $i)) * 1 . ' ' . $sizes[$i];
	}

    public static function format_duration($msecs)
    {
        $n = intval($msecs);

        if (strlen($msecs) <= 0 || $n <= 0)
            return "--:--";

        $n = $n / 1000;
        $hours = $n / 3600;
        $remainder = $n % 3600;
        $minutes = $remainder / 60;
        $seconds = $remainder % 60;

        if (intval($hours) > 0)
        {
            return sprintf("%d:%02d:%02d", $hours, $minutes, $seconds);
        }
        else
        {
            return sprintf("%02d:%02d", $minutes, $seconds);
        }
    }

    ///////////////////////////////////////////////////////////////////////

    public static function encode_user_data($a, $b = null)
    {
        $media_url = null;
        $user_data = null;

        if (is_array($a) && is_null($b))
        {
            $media_url = '';
            $user_data = $a;
        }
        else
        {
            $media_url = $a;
            $user_data = $b;
        }

        if (!is_null($user_data))
            $media_url .= '||' . json_encode($user_data);

        return $media_url;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function decode_user_data($media_url_str, &$media_url, &$user_data)
    {
        $idx = strpos($media_url_str, '||');

        if ($idx === false)
        {
            $media_url = $media_url_str;
            $user_data = null;
            return;
        }

        $media_url = substr($media_url_str, 0, $idx);
        $user_data = json_decode(substr($media_url_str, $idx + 2));
    }

    ///////////////////////////////////////////////////////////////////////

    public static function create_regular_folder_range($items,
        $from_ndx = 0, $total = -1, $more_items_available = false)
    {
        if ($total === -1)
            $total = $from_ndx + count($items);

        if ($from_ndx >= $total)
        {
            $from_ndx = $total;
            $items = array();
        }
        else if ($from_ndx + count($items) > $total)
        {
            array_splice($items, $total - $from_ndx);
        }

        return array
        (
            PluginRegularFolderRange::total => intval($total),
            PluginRegularFolderRange::more_items_available => $more_items_available,
            PluginRegularFolderRange::from_ndx => intval($from_ndx),
            PluginRegularFolderRange::count => count($items),
            PluginRegularFolderRange::items => $items
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public static function http_get_document($url, $opts = null)
    {
        $ch = curl_init();

		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 	0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 	0);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT,    30);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 	true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,    true);
        curl_setopt($ch, CURLOPT_TIMEOUT,           30);
        curl_setopt($ch, CURLOPT_USERAGENT,         'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36');
        curl_setopt($ch, CURLOPT_URL,               $url);
		curl_setopt($ch, CURLOPT_ENCODING,          'gzip,deflate');

        if (isset($opts))
        {
            foreach ($opts as $k => $v) {
				if ($k == 'getHttpCode')
					$getHttpCode = true;
                else
                    curl_setopt($ch, $k, $v);
            }
        }

        hd_print("HTTP fetching '$url'...");

        $content = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if($content === false){
            $err_msg['Error:'] = "HTTP error $http_code (" . curl_error($ch) . ')';
			if (!isset($silent))
				echo($err_msg['Error:']);
        }
        if ($http_code != 200){
            $err_msg['Request:'] = "HTTP request failed ($http_code): " . self::http_status_code_to_string($http_code);
			if (!isset($silent))
				echo($err_msg['Request:']);
        }
        curl_close($ch);
		if ((!isset($err_msg))&&(!isset($silent)))
			echo("HTTP OK ($http_code)");
		if (isset($getHttpCode)){
			$err_msg['httpCode'] = $http_code;
			$err_msg['status'] = $content;
			return $err_msg;
		}
        return $content;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function http_post_document($url, $post_data, $opts=null)
    {
        if (!$opts)
            $opts = array();
        $opts[CURLOPT_POST] = true;
        $opts[CURLOPT_POSTFIELDS] = $post_data;
        return self::http_get_document($url, $opts);
    }

    ///////////////////////////////////////////////////////////////////////

    public static function parse_xml_document($doc)
    {
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        return $xml;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function make_json_rpc_request($op_name, $params)
    {
        static $request_id = 0;

        $request = array
        (
            'jsonrpc' => '2.0',
            'id' => ++$request_id,
            'method' => $op_name,
            'params' => $params
        );

        return $request;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function append_dune_auth($url)
    {
        $sep = FALSE === strpos($url, '?') ? '?' : '&';
        $res = file_get_contents(self::DUNE_AUTH_PATH);
        $data = $res === false ? '' : $res;
        return "$url${sep}$data";
    }

    ///////////////////////////////////////////////////////////////////////////

    public static function load_properties_file($path, $map=null)
    {
        if (!is_object($map))
            $map = (object) array();

        if ($path && is_file($path))
        {
            foreach (file($path) as $line)
            {
                $pos = strpos($line, "=");
                if ($pos === false)
                    continue;

                $key = trim(substr($line, 0, $pos));
                if ($key != '' && $key[0] != '#')
                    $map->$key = trim(substr($line, $pos + 1));
            }
        }

        return $map;
    }

    ///////////////////////////////////////////////////////////////////////////

    public static function get_ip_addr()
    {
        static $ip_addr = null;

        if (is_null($ip_addr))
        {
            $ip_addr = shell_exec(
                'ifconfig eth0 | head -2 | tail -1 | sed "s/^.*inet addr:\([^ ]*\).*$/\1/"');

            $ip_addr = trim($ip_addr);

            hd_print("IP Address: '$ip_addr'");
        }

        return $ip_addr;
    }

    ///////////////////////////////////////////////////////////////////////////

    public static function get_mac_addr()
    {
        static $mac_addr = null;

        if (is_null($mac_addr))
        {
            if (HD::is_apk()) {
                $mac_addr = strtoupper(
                    trim(file_get_contents("/tmp/run/dune_mac.txt")));
            } else {
                $mac_addr = shell_exec(
                    'ifconfig  eth0 | head -1 | sed "s/^.*HWaddr //"');
                $mac_addr = trim($mac_addr);
            }

            hd_print("MAC Address: '$mac_addr'");
        }

        return $mac_addr;
    }

    ///////////////////////////////////////////////////////////////////////////

    public static function get_serial_number()
    {
        static $serial_number = null;

        if (is_null($serial_number))
        {
            $serial_number = shell_exec(
                'grep "serial_number:" /tmp/sysinfo.txt | sed "s/^.*: *//"');

            $serial_number = trim($serial_number);
            hd_print("Serial number: '$serial_number'");
        }

        return $serial_number;
    }

    ///////////////////////////////////////////////////////////////////////////

    public static function escape_xml_string($str)
    {
        $str = strval($str);
        $len = strlen($str);
        $out = '';
        for ($i = 0; $i < $len; $i++)
        {
            if ($str[$i] == '&')
                $out .= '&amp;';
            else if ($str[$i] == '<')
                $out .= '&lt;';
            else if ($str[$i] == '>')
                $out .= '&gt;';
            else if ($str[$i] == '"')
                $out .= '&quot;';
            else
                $out .= $str[$i];
        }
        return $out;
    }

    private static function get_month_key($n)
    {
        return "formatting_month_$n";
    }

    public static function format_date_time_date_ext($tm)
    {
        $lt = localtime($tm);
        $mon_key = self::get_month_key($lt[4]);
        return sprintf("%02d <key_global>%s</key_global> %04d",
            $lt[3], $mon_key, $lt[5] + 1900);
    }

    public static function format_date_time_time($tm, $with_sec = false)
    {
        $format = '%H:%M';
        if ($with_sec)
            $format .= ':%S';
        return strftime($format, $tm);
    }

    public static function print_backtrace()
    {
        hd_print('Back trace:');
        foreach (debug_backtrace() as $f)
        {
            hd_print(
                '  - ' . $f['function'] . 
                ' at ' . $f['file'] . ':' . $f['line']);
        }
    }

    public static function path_basename($str)
    {
        return preg_replace('/^.*\//', '', $str);
    }

    public static function path_dirname($str)
    {
        $str = rtrim($str, '/');
        $ndx = strrpos($str, '/');
        if ($ndx === false)
            return '';
        if ($ndx == 0)
            return '/';
        return substr($str, 0, $ndx);
    }

    public static function http_status_code_to_string($code)
    {
        // Source: http://en.wikipedia.org/wiki/List_of_HTTP_status_codes

        switch( $code )
        {
            // 1xx Informational
            case 100: $string = 'Continue'; break;
            case 101: $string = 'Switching Protocols'; break;
            case 102: $string = 'Processing'; break; // WebDAV
            case 122: $string = 'Request-URI too long'; break; // Microsoft

            // 2xx Success
            case 200: $string = 'OK'; break;
            case 201: $string = 'Created'; break;
            case 202: $string = 'Accepted'; break;
            case 203: $string = 'Non-Authoritative Information'; break; // HTTP/1.1
            case 204: $string = 'No Content'; break;
            case 205: $string = 'Reset Content'; break;
            case 206: $string = 'Partial Content'; break;
            case 207: $string = 'Multi-Status'; break; // WebDAV

            // 3xx Redirection
            case 300: $string = 'Multiple Choices'; break;
            case 301: $string = 'Moved Permanently'; break;
            case 302: $string = 'Found'; break;
            case 303: $string = 'See Other'; break; //HTTP/1.1
            case 304: $string = 'Not Modified'; break;
            case 305: $string = 'Use Proxy'; break; // HTTP/1.1
            case 306: $string = 'Switch Proxy'; break; // Depreciated
            case 307: $string = 'Temporary Redirect'; break; // HTTP/1.1

            // 4xx Client Error
            case 400: $string = 'Bad Request'; break;
            case 401: $string = 'Unauthorized'; break;
            case 402: $string = 'Payment Required'; break;
            case 403: $string = 'Forbidden'; break;
            case 404: $string = 'Not Found'; break;
            case 405: $string = 'Method Not Allowed'; break;
            case 406: $string = 'Not Acceptable'; break;
            case 407: $string = 'Proxy Authentication Required'; break;
            case 408: $string = 'Request Timeout'; break;
            case 409: $string = 'Conflict'; break;
            case 410: $string = 'Gone'; break;
            case 411: $string = 'Length Required'; break;
            case 412: $string = 'Precondition Failed'; break;
            case 413: $string = 'Request Entity Too Large'; break;
            case 414: $string = 'Request-URI Too Long'; break;
            case 415: $string = 'Unsupported Media Type'; break;
            case 416: $string = 'Requested Range Not Satisfiable'; break;
            case 417: $string = 'Expectation Failed'; break;
            case 422: $string = 'Unprocessable Entity'; break; // WebDAV
            case 423: $string = 'Locked'; break; // WebDAV
            case 424: $string = 'Failed Dependency'; break; // WebDAV
            case 425: $string = 'Unordered Collection'; break; // WebDAV
            case 426: $string = 'Upgrade Required'; break;
            case 449: $string = 'Retry With'; break; // Microsoft
            case 450: $string = 'Blocked'; break; // Microsoft

            // 5xx Server Error
            case 500: $string = 'Internal Server Error'; break;
            case 501: $string = 'Not Implemented'; break;
            case 502: $string = 'Bad Gateway'; break;
            case 503: $string = 'Service Unavailable'; break;
            case 504: $string = 'Gateway Timeout'; break;
            case 505: $string = 'HTTP Version Not Supported'; break;
            case 506: $string = 'Variant Also Negotiates'; break;
            case 507: $string = 'Insufficient Storage'; break; // WebDAV
            case 509: $string = 'Bandwidth Limit Exceeded'; break; // Apache
            case 510: $string = 'Not Extended'; break;

            // Unknown code:
            default: $string = 'Unknown'; break;
        }
        return $string;
    }

    ///////////////////////////////////////////////////////////////////////
    
	public static function str_remove_tag($str){
		$str = str_replace(array("&nbsp;","&thinsp;")," ",$str);
		$str = html_entity_decode(strip_tags($str), ENT_QUOTES,'UTF-8');
		$str = preg_replace('/^\:/', '', $str);
		$str = preg_replace('/ {2,}/', ' ', $str);
		$str = preg_replace('/	{2,}/', ' ', $str);
		$str = preg_replace("/\n{2,}/", "\n", $str);
		$str = trim($str);
		return $str;
	}

	public static function mb_str_split($str, $num=1){
		if (preg_match_all("#.{".$num."}#uis", $str, $arr)){
			foreach ($arr[0] as $v)
				$out[]= trim($v);
		}else
			$out[]= $str;
		return $out;
	}

	public static function get_data_path($reset=null) {
        return DuneSystem::$properties['data_dir_path'];
    }

	public static function getItemsArr($path, $macAdr=null) {
		$items = array();
		if ($macAdr==true)
			$path .= '_' . str_replace(":", "", self::get_mac_addr());
		if ($path=='data_dir_path')
			$link = DuneSystem::$properties['data_dir_path'] . '/'. $path;
		else
			$link = self::get_data_path() . '/'. $path;
		if (file_exists($link))
			$items = unserialize(file_get_contents($link));
		return $items;
	}
	public static function saveItemsArr($path, $items, $macAdr=null){
		if ($macAdr==true)
			$path .= '_' . str_replace(":", "", self::get_mac_addr());
		if ($path=='data_dir_path')
			$link = DuneSystem::$properties['data_dir_path'] . '/'. $path;
		else
			$link = self::get_data_path() . '/'. $path;
		if (file_put_contents ($link, serialize($items)) !== false)
			return true;
		else
			return false;
	}

	public static function getItemStr($path, $macAdr=null) {
		$item = '';
		if ($macAdr == true)
			$path .= '_' . str_replace(":", "", self::get_mac_addr());
		$link = self::get_data_path() . "/$path";
		if (file_exists($link))
			$item = file_get_contents($link);
		return $item;
	}
	public static function saveItemStr($path, $item, $macAdr=null) {
		if ($macAdr == true)
			$path .= '_' . str_replace(":", "", self::get_mac_addr());
		$link = self::get_data_path() . "/$path";
		if (file_put_contents ($link, $item) !== false)
			return true;
		else
			return false;
	}

	public static function get_ip_address(){
        static $ip_address = null;
        if (is_null($ip_address)){
            preg_match_all('/Link encap:Ethernet.*?inet'.(false ? '6?' : '').' addr: ?([^ ]+)/ms', `ifconfig`, $ips);
			if ($ips[1][0]!= '127.0.0.1')
				$ip_address = $ips[1][0];
			else if ($ips[1][1]!= '127.0.0.1')
				$ip_address = $ips[1][1];
        }
        return $ip_address;
    }

	public static function get_platform_kind(){
		static $platform_kind = null;
		if (is_null($platform_kind)){
			$ini_arr = parse_ini_file('/tmp/run/versions.txt');
			if (preg_match('|87\d\d|', $ini_arr['platform_kind']))
				$platform_kind = '87xx';
			if (preg_match('|864\d|', $ini_arr['platform_kind']))
				$platform_kind = '864x';
			if (preg_match('|867\d|', $ini_arr['platform_kind']))
				$platform_kind = '867x';
			if (preg_match('|865\d|', $ini_arr['platform_kind']))
				$platform_kind = '865x';
			if (preg_match('|android|', $ini_arr['platform_kind']))
				$platform_kind = 'android';
		}
		return $platform_kind;
	}

	public static function isDomainAvailible($domain, $timeout=null){
		if ($timeout==false)
			$timeout=1;
		$curlInit = curl_init($domain);
		curl_setopt($curlInit,CURLOPT_CONNECTTIMEOUT,$timeout);
		if (!preg_match('|/echo$|', $domain)){
			curl_setopt($curlInit,CURLOPT_HEADER,true);
			curl_setopt($curlInit,CURLOPT_NOBODY,true);
		}
		curl_setopt($curlInit,CURLOPT_RETURNTRANSFER,true);
		$response = curl_exec($curlInit);
		curl_close($curlInit);
		if ((preg_match('|/echo$|', $domain))&&($response))
			return $response;
		if ($response){
			if (!preg_match('/404/', $response))
				return true;
		}
		return false;
	}

	public static function isServerAvailable ($ip=null, $port=null){
		if ($ip == true){
			$ip = str_replace(array('https://', 'http://'), '', $ip);
			if ($port == true)
				$hosts[] = "$ip:$port";
			else
				$hosts[] = $ip;
		}else if (($port == true)&&(preg_match("/(\d{1,3}\.\d{1,3}\.\d{1,3})\.\d{1,3}$/", self::get_ip_address(), $match)))
			for ($i = 1; $i<255; $i++)
				$hosts[] = $match[1] . ".$i:$port";
		else
			return false;
		$timeout = 1;
		$trueArr = $status = $sockets = array();
		foreach ($hosts as $id => $host) {
			$s = @stream_socket_client ($host, $errno, $errstr, $timeout, STREAM_CLIENT_ASYNC_CONNECT|STREAM_CLIENT_CONNECT);
			if($s)
				$sockets[$id] = $s;
		}
		while (count($sockets)) {
			$read = $write = $sockets;
			hd_silence_warnings();
			$n = stream_select ($read, $write, $e = null, $timeout);
			hd_restore_warnings();
			if ($n > 0) {
				foreach ($read as $r) {
					$id = array_search($r, $sockets);
					$data = fread($r, 8192);
					if (strlen($data) == 0) {
						fclose($r);
						unset ($sockets[$id]);
					}else 
						$status[$id] = 1;
				}
				foreach ($write as $w) {
					$id = array_search($w, $sockets);
					hd_silence_warnings();
					@fwrite ($w, "HEAD / HTTP/1.0\r\nHost: " . $hosts[$id] . "\r\n\r\n");
					hd_restore_warnings();
				}
			}else
				break;
		}
		if (count($status) > 0)
			foreach ($status as $k => $v)
				$trueArr [$hosts[$k]]= $hosts[$k];
		if ($ip== true){
			if ($port == true){
				if(isset($trueArr ["$ip:$port"]))
					return true;
				else
					return false;
			}else{
				if(isset($trueArr [$ip]))
					return true;
				else
					return false;
			}
		}
		return $trueArr;
	}
}

///////////////////////////////////////////////////////////////////////////
?>
