<?php

require_once 'lib/user_input_handler_registry.php';
require_once 'lib/abstract_preloaded_regular_screen.php';

class SeasonsScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'seasons';

    // EEE TODO: copy-n-paste

    const PAGE_SIZE = 9;

    const MOV_ICON_W = 52;
    const MOV_ICON_H = 70;
    const MOV_ICON_DX = 13;

    const OTHER_ICON_W = 48;
    const OTHER_ICON_H = 48;
    const OTHER_ICON_DX = 15;

    public static function get_media_url_str($movie_id)
    {
        $arr = array(
            'screen_id' => self::ID,
            'movie_id' => $movie_id,
        );
        return MediaURL::encode($arr);
    }

    ///////////////////////////////////////////////////////////////////////

    public function __construct()
    {
        parent::__construct(self::ID);

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_handler_id()
    {
        return self::ID;
    }

    public function custom_action($control_id,
        $params=null, $timeout_ms=null, $show_dialog_delay_ms=null)
    {
        return UserInputHandlerRegistry::create_action($this,
            $control_id, $params, $timeout_ms, $show_dialog_delay_ms);
    }

    ///////////////////////////////////////////////////////////////////////

    // EEE TODO: copy-n-paste
    public function get_folder_view(MediaURL $media_url, &$plugin_cookies)
    {
        $this->set_folder_views(null);
        return parent::get_folder_view($media_url, $plugin_cookies);
    }

    // EEE TODO: copy-n-paste
    protected function do_get_folder_views(&$plugin_cookies)
    {
        $loading_url = "plugin_file://%shell_ext%/icons/movies_small.png";
        $load_failed_url = "plugin_file://%shell_ext%/icons/no_movie_poster.png";

        $height = 700;

        return array(
            array
            (
                PluginRegularFolderView::view_params => array_merge(
                    array(
                        ViewParams::num_cols => 1,
                        ViewParams::num_rows => 11,
                        ViewParams::paint_details => false,

                        ViewParams::scroll_animation_enabled => true,

                        ViewParams::content_box_y => 250,
                        ViewParams::content_box_height => $height,
                    )),

                PluginRegularFolderView::base_view_item_params => array(
                    ViewItemParams::item_padding_top => 0,
                    ViewItemParams::item_padding_bottom => 0,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::item_caption_width => 1570,
                    ViewItemParams::item_caption_font_size => FONT_SIZE_SMALL,
                    ViewItemParams::icon_dx => self::OTHER_ICON_DX,
                    ViewItemParams::icon_width => 38,
                    ViewItemParams::icon_height => 38,
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array
                (
                    ViewItemParams::icon_path => $loading_url,
                ),

                PluginRegularFolderView::load_failed_view_item_params => array
                (
                    ViewItemParams::icon_path => $load_failed_url,
                    ViewItemParams::icon_width => self::MOV_ICON_W,
                    ViewItemParams::icon_height => self::MOV_ICON_H,
                    ViewItemParams::icon_dx => self::MOV_ICON_DX,
                ),
            ));
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        return array(
            GUI_EVENT_KEY_ENTER => $this->custom_action('enter'),
            GUI_EVENT_KEY_INFO => $this->custom_action('info'),
            GUI_EVENT_MENU_PLAYBACK_FINISH =>
                $this->custom_action("menu_playback_finish"),
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(
        MediaURL $media_url, &$plugin_cookies)
    {
        $items = array();

        $mts = MainHelper::$state->nav_mts;
        $mts->ensure_torrents_info(false);

        $url = "plugin_file://%shell_ext%/icons/folder_small.png";

        $seasons = $mts->get_torrent_seasons();
        foreach ($seasons as $s)
        {
            $vip = array(ViewItemParams::icon_path => $url);
            $vip[ViewItemParams::item_caption_dy] = -2;
            $vip[ViewItemParams::item_caption_sel_dy] = -2;

            $caption = UrlHelper::get_season_caption($s);
            $murl = self::get_item_media_url($s);

            $items[] = array(
                PluginRegularFolderItem::media_url => $murl,
                PluginRegularFolderItem::caption => $caption,
                PluginRegularFolderItem::view_item_params => $vip,
            );
        }

        return $items;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function get_item_media_url($ndx)
    {
        return $ndx;
    }

    private function menu_playback_finish($user_input)
    {
        return TorrentHelper::menu_playback_finish($user_input);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('Seasons screen: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        $st = MainHelper::$state;
        $mts = $st->nav_mts;
        $m = $mts->get_movie();

        $s = (int) HD::get($user_input, 'chosen_media_url',
            HD::get($user_input, 'selected_media_url', -1));

        if ($user_input->control_id == 'enter')
        {
            $chosen_only = $s >= 1;
            $stack = "S2:$s;";
            $media_url_str = TorrentsScreen::get_media_url_str(
                $m->id, $s, -1, $chosen_only, $stack);

            $sel_id = null;
            $post_action = null;

            if (isset($user_input->torrent_sel_id))
            {
                $sel_id = $user_input->torrent_sel_id;
                $enter_torrent_action =
                    HD::get($user_input, 'enter_torrent_action');
                if ($enter_torrent_action)
                    $post_action = json_decode($enter_torrent_action, true);
            }

            $st->enter_season($s);
            return ActionFactory::open_folder_ext($media_url_str,
                UrlHelper::get_season_caption($s),
                null, $sel_id, $post_action);
        }

        if ($user_input->control_id == 'menu_playback_finish')
            return $this->menu_playback_finish($user_input);

        return null;
    }
}
