<?php

class StorageHelper
{
    const STORAGE_LIST_FILE = '/tmp/run/storage_list.xml';

    public $storages = array();

    public function reload_storage_list($silent=false)
    {
        $this->storages = array();
        if (!$silent)
            hd_print('Loading storage list.');

        $path = StorageHelper::STORAGE_LIST_FILE;

        $doc = file_get_contents($path);
        if ($doc === false)
        {
            hd_print("Failed to load $path.");
            return;
        }

        $xml = simplexml_load_string($doc);
        if ($xml === false)
        {
            hd_print("Error parsing $path.");
            return;
        }

        if (!isset($xml->storages)) {
            hd_print("Warning: root element not 'storages' in $path.");
            return;
        }

        foreach ($xml->storages->children() as $storage)
        {
            $elt_name = $storage->getName();
            if ($elt_name != 'storage')
                continue;

            if (intval($storage->is_persistfs_capable) == 1)
                continue;
            if (!isset($storage->name))
                continue;

            $id = strval($storage->id);
            $kind = strval($storage->kind);
            $name = strval($storage->name);
            $uuid = strval($storage->uuid);
            $mount_path = strval($storage->mount_path);

            $arr = array(
                'id' => $id,
                'kind' => $kind,
                'name' => $name,
                'uuid' => $uuid,
                'mount_path' => $mount_path
            );

            if (isset($storage->label))
                $arr['label'] = strval($storage->label);
            if (isset($storage->user_name))
                $arr['user_name'] = strval($storage->user_name);
            if (isset($storage->user_caption))
                $arr['user_caption'] = strval($storage->user_caption);

            if (!$silent)
            {
                hd_print("Added storage: $id (kind: $kind, ".
                    "name: $name, uuid: $uuid, mount_path: $mount_path)");
            }

            $this->storages[] = (object) $arr;
        }
    }

    public function get_dl_storage_name()
    {
        $this->reload_storage_list(true);
        foreach ($this->storages as $storage) {
            if ($storage->mount_path != '/tmp/mnt/D')
                continue;

            if (!is_dir("/tmp/mnt/D/dune_downloads")) {
                hd_print("No dune_downloads folder on main storage");
            }

            return $storage->name;
        }
        return null;
    }

}

?>
