<?php

class SwitchDialog extends ProgressDialogManager
{
    const ID = 'switch_dialog';

    private $progress;

    private $special_order = 0;
    private $prepare_cancel_allowed = 0;

    public $opened = 0;
    private $open_tm = 0;

    private $confirmed = false;
    private $visual_torrent_id = null;
    private $visual_torrent_index = -1;
    public $last_stop_position = -1;
    public $last_stop_series_ndx = -1;

    public function __construct()
    {
        $this->progress = MainHelper::$progress;
    }

    public function open($special_order=true, $post_action=null)
    {
        if ($this->opened)
            hd_print("SwitchDialog E: already opened");

        $this->special_order = $special_order;
        $this->prepare_cancel_allowed = !isset($post_action);

        $this->opened = 1;
        $this->open_tm = microtime(true);
        $this->confirmed = false;
        $this->last_stop_position = -1;
        $this->last_stop_series_ndx = -1;
        $this->visual_torrent_id = null;
        $this->visual_torrent_index = -1;

        $p = $this->progress;
        if ($p->opened && $p->manager === $this) {
            hd_print("SwitchDialog W: progress already opened");
        }
        else if ($p->opened && $p->manager) {
            hd_print("SwitchDialog W: progress already opened by another manager");
        }

        $this->set_manager();
        return $p->open($post_action);
    }

    public function set_manager() {
        $this->progress->set_manager($this);
        $this->progress->set_dialog_params(1440, 1);
        $this->progress->set_events(
            array(GUI_EVENT_KEY_ENTER),
            array('switch_torrent', 'apply_series', 'switch_open_torrent'));
        $this->update_dialog();
        $this->update_timer();
    }

    ///////////////////////////////////////////////////////////////////////

    private function update_timer() {
        $this->progress->set_timer_delay($this->confirmed ? 4900 : 950);
    }

    private function update_dialog() {
        $this->progress->set_labels($this->get_labels());
    }

    private function get_switch_position() {
        $pb = MainHelper::$playback;
        $playing_pos = $pb->get_last_playing_position();
        $stop_pos = $this->last_stop_position;
        $pos = $stop_pos > 0 && !$pb->is_started() ?
            $stop_pos : $playing_pos;

        if ($pos < 0)
            $pos = 0;
        hd_print("DEBUG using position $pos ".
            "(playing $playing_pos, stop $stop_pos)");
        return $pos;
    }

    public function switch_torrent($user_input)
    {
        $st = MainHelper::$state;
        $mts = $st->play_mts;
        $tfs = $st->play_tfs ? $st->play_tfs : $st->prev_play_tfs;
        $m = $mts->get_movie();
        $torrent_id = $tfs->get_torrent_id();
        $total = count($mts->get_torrents());

        $next_tid = $torrent_id;
        for (;;)
        {
            $t = $mts->get_next_torrent($next_tid, $this->special_order);
            if (!$t) {
                // Should not happen?..
                hd_print("Switch dialog: no more torrents (1)");
                break;
            }
            $next_tid = $t->id;
            $ndx = $mts->get_torrent_index($next_tid, $this->special_order);
            if ($ndx < 0) {
                // Should not happen?..
                hd_print("Switch dialog: no more torrents (2)");
                break;
            }

            $next_tuid = $t->uid;

            $pub_ndx = $mts->get_torrent_public_index($next_tid);

            hd_print("DEBUG next torrent [$pub_ndx of $total]: ".
                "$t->name | $t->size ($t->S/$t->L)");

            $this->visual_torrent_id = $next_tid;
            $this->visual_torrent_index = $pub_ndx;
            $this->update_dialog();

            $pos = $this->get_switch_position();

            $stack = ''; // XXX TODO: revise stack usage
            $res = TorrentHelper::open_torrent(
                1, $pos, $m->id, $mts->get_season(),
                $next_tid, $next_tuid, $stack, null, null, null);
            if (is_array($res) || !isset($res))
                return $res;
        }

        return null;
    }

    private function switch_open_torrent($user_input)
    {
        $mts = MainHelper::$state->play_mts;
        $m = $mts->get_movie();

        $torrent_id = $user_input->torrent_id;
        $e = $user_input->e;

        $t = $mts->get_torrent_by_id($torrent_id);

        $pub_ndx = $mts->get_torrent_public_index($torrent_id);
        $total = count($mts->get_torrents());

        hd_print("DEBUG opening torrent [$pub_ndx of $total]: ".
            "$t->name | $t->size ($t->S/$t->L)");

        $this->confirmed = true;
        $this->visual_torrent_id = $torrent_id;
        $this->visual_torrent_index = $pub_ndx;
        $this->update_dialog();

        $pos = $this->get_switch_position();

        $stack = ''; // XXX TODO: revise stack usage
        $res = TorrentHelper::open_torrent(
            1, $pos, $m->id, $mts->get_season(),
            $torrent_id, $t->uid, $stack, null, null, null);

        $ea = TorrentHelper::open_torrent_error_action($res, true);
        if ($ea)
            return $ea;

        if (!isset($res) || is_array($res)) {
            return $this->progress->change_behaviour(
                $this->progress->reset_controls($res));
        }

        return $this->progress->close_and_run($user_input,
            ActionFactory::show_error(true, "Illegal state 10"));
    }

    public function playback_stop(&$user_input)
    {
        if (!$this->opened || !$this->confirmed)
            return $this->progress->close_and_run(null);

        $pos = HD::get($user_input, 'plugin_vod_stop_position', 0);
        if ($pos > 0) {
            hd_print("Switch dialog: keeping position from 'stop' action: $pos");
        } else {
            $pos = MainHelper::$playback->get_last_playing_position();
            hd_print("Switch dialog: keeping last playing position: $pos");
        }
        $this->last_stop_position = $pos;

        // TODO: revise
/*
        $pb = MainHelper::$playback;
        $ser_ndx = HD::get($user_input, 'plugin_vod_series_ndx', -1);
        if ($ser_ndx >= 0)
            $pb->assign_episode_by_series_ndx($ser_ndx);

        $mts = MainHelper::$state->play_mts;
        if ($pb->get_episode() != $mts->get_episode())
        {
            hd_print("Switch dialog: changing mts episode: ".
                $mts->get_episode() . " -> " . $pb->get_episode());
            $mts->set_episode($pb->get_episode());
        }
*/

        return $this->switch_torrent($user_input);
    }

    ///////////////////////////////////////////////////////////////////////

    private function get_message()
    {
        if (!$this->confirmed || !$this->visual_torrent_id)
        {
            if ($this->prepare_cancel_allowed)
                return T::t('title_press_return_to_cancel');

            if (!$this->visual_torrent_id)
                return T::t('title_preparing');
        }

        $mts = MainHelper::$state->play_mts;
        $inf = $mts->get_torrent_info($this->visual_torrent_id);
        $total = count($mts->get_torrents($mts->get_season()));
        $caption = $inf->get_caption($mts->get_season());
        $t = $mts->get_torrent_by_id($this->visual_torrent_id);
        $extra_text = HD::get($t, 'extra_item');
        if ($extra_text)
            $caption .= " | " . $extra_text;

        return T::t3('msg_trying_next__3',
            $this->visual_torrent_index, $total, $caption);
    }

    public function get_labels() {
        $labels = array();
        $title = $this->special_order ?
            T::t('title_switching_video_stream') :
            T::t('title_opening_video_stream');
        $labels[] = new ProgressDialogLabel(
            $title, HALIGN_CENTER, "14");
        $labels[] = new ProgressDialogLabel($this->get_message());
        return $labels;
    }

    public function set_confirmed()
    {
        if ($this->confirmed)
            return true;

        if (!$this->visual_torrent_id) {
            $st = MainHelper::$state;
            $tfs = $st->play_tfs ? $st->play_tfs : $st->prev_play_tfs;
            $torrent_id = $tfs->get_torrent_id();

            $mts = MainHelper::$state->play_mts;
            $t = $mts->get_next_torrent($torrent_id, $this->special_order);
            if (!$t)
                return false; // Should not happen

            $this->visual_torrent_id = $t->id;
            $this->visual_torrent_index = $mts->get_torrent_public_index($t->id);
        }

        $this->confirmed = true;
        return true;
    }

    private function confirm()
    {
        $this->set_confirmed();

        $this->update_timer();
        $this->update_dialog();

        return ActionFactory::stop_playback(false);
    }

    public function close_allowed()
    {
        return $this->confirmed || $this->prepare_cancel_allowed;
    }

    public function on_close($user_input)
    {
        $atd = MainHelper::$add_torrent_dialog;
        if ($atd->opened)
            $atd->on_close($user_input);
        $this->opened = 0;
        MainHelper::$playback->switch_canceled();
    }

    public function on_event($ev, $user_input)
    {
        if ($ev == 'enter') {
            if ($this->confirmed)
                return null;
            hd_print("Switch dialog: confirmed by ENTER");
            return $this->confirm();
        }
        if ($ev == 'switch_torrent')
            return $this->switch_torrent($user_input);
        if ($ev == 'apply_series')
            return MainHelper::$add_torrent_dialog->apply_series($user_input);
        if ($ev == 'switch_open_torrent')
            return $this->switch_open_torrent($user_input);
        return null;
    }

    public function on_timer($user_input)
    {
        $atd = MainHelper::$add_torrent_dialog;

        $tm = microtime(true);
        hd_print("Switch dialog: timer ($this->confirmed, $atd->opened)");

        if (!$this->confirmed && $tm - $this->open_tm > 1.9)
        {
            hd_print("Switch dialog: confirmed by timer");
            return $this->confirm();
        } 

        if ($atd->opened)
            return $atd->on_timer($user_input);

        return null;
    }
}

?>
