<?php

class TorrentFiles
{
    private $switch_mode;
    private $switch_position;
    private $movie_id;
    private $season;
    private $torrent_id;
    private $torrent_uid;
    private $group_id;
    private $hash;
    private $stack;

    private $file_stats = array();
    private $series = array();

    public function __construct($switch_mode, $switch_position,
        $movie_id, $season, $torrent_id, $torrent_uid, $hash, $stack)
    {
        $this->switch_mode = $switch_mode;
        $this->switch_position = $switch_position;
        $this->movie_id = $movie_id;
        $this->season = $season;
        $this->torrent_id = $torrent_id;
        $this->torrent_uid = $torrent_uid;
        $this->group_id = TorrentHelper::get_group_id($torrent_id);
        $this->hash = $hash;
        $this->stack = $stack;
    }

    public function set_files($file_stats)
    {
        $this->file_stats = $file_stats;
        $this->series =
            TorrServer::do_get_series_list($this->hash, $this->file_stats);
        $this->finalize_titles();
    }

    private function finalize_titles()
    {
        if (count($this->series) <= 1)
            return;

        $min_depth = 99;
        $last_path = null;

        foreach ($this->series as $path => $ser)
        {
            if ($last_path) {
                $min_depth = self::get_common_depth(
                    $path, $last_path, $min_depth);
            }
            $last_path = $path;
        }

        $pos = 0;
        for ($d = 0; $d < $min_depth; $d++)
            $pos = strpos($last_path, '/', $pos) + 1;

        $prefix = substr($last_path, 0, $pos);

        foreach ($this->series as $path => &$ser)
            $ser['title'] = substr($path, $pos);
        unset($ser);
    }

    private static function get_common_depth($path1, $path2, $max_depth)
    {
        $depth = 0;
        for ($pos = 0; $pos !== false; $depth++) {
            if ($depth >= $max_depth)
                break;

            $p1 = strpos($path1, '/', $pos);
            $p2 = strpos($path2, '/', $pos);
            if ($p1 !== $p2)
                break;

            if (!$p1) {
                if (substr($path1, $pos) != substr($path2, $pos))
                    break;
                $pos = false;
            } else {
                $s1 = substr($path1, $pos, $p1 - $pos);
                $s2 = substr($path2, $pos, $p2 - $pos);
                if ($s1 != $s2)
                    break;
                $pos = $p1 + 1;
            }
        }
        return $depth;
    }

    public function get_torrent_files_vod_movie($stack,
        $autostart, $sel_id=null, $initial_position=-1)
    {
        $st = MainHelper::$state;
        $mts = $st->play_mts;
        $m = $mts->get_movie();

        hd_print("DEBUG get_torrent_files_vod_movie(): ".
            "stack $stack, autostart $autostart, inipos $initial_position");

        $vod_movie = TorrentHelper::get_vod_movie_base(
            $this->torrent_id, $this->torrent_uid,
            $initial_position, $autostart);

        $vod_movie->id = "F.$vod_movie->id";

        $pb = MainHelper::$playback;
        $pb->set_playback_parameters($vod_movie->id, $stack, 0, -1, -1);

        $th = MainHelper::$torrent_history;

        $path_by_sndx = array();

        $sndx = 0;
        foreach ($this->series as $path => $ser)
        {
            $recent_uid = $th->make_file_uid(
                $m->id, $this->torrent_uid, $ser['id']);
            $e = HD::get($ser, 'e', -1);
            $s = HD::get($ser, 's', -1);

            $mdb_info = ActionFactory::file_movie_info(
                $m->id, $m->title, $m->poster_url, $s, $e);
            $ser_title = $ser['title'];

            $pl_url = $ser['playback_url'];
            $pl_url .= "|||dune_params|||http_online_subs:1,file_id:$path";

            $vod_movie->add_series_data($ser['id'],
                $ser_title, $pl_url, true,
                $recent_uid, $mdb_info);

            $path_by_sndx[$sndx] = $path;
            $sndx++;
        }

        $pb->set_episode_by_series_ndx(array());
        $pb->set_path_by_series_ndx($path_by_sndx);

        return $vod_movie;
    }

    public function get_ser_by_playback_url($url)
    {
        $pos = strpos($url, '|||dune_params|||');
        if ($pos)
            $url = substr($url, 0, $pos);

        foreach ($this->series as $ser)
        {
            if ($ser['playback_url'] == $url)
                return $ser;
        }
        return null;
    }

    public function find_suitable_episode($s, $e) {
        foreach ($this->series as $ser) {
            if (!isset($ser['e']))
                continue;
            
            if ($ser['s'] == $s && $ser['e'] == $e)
                return $ser['id'];
        }
        return null;
    }

    public function get_hash() {
        return $this->hash;
    }

    public function &get_series()
    {
        return $this->series;
    }

    public function find_series_by_id($id)
    {
        foreach ($this->series as $path => $ser) {
            if ($ser['id'] == $id)
                return $path;
        }
        return null;
    }

    public function get_switch_mode()
    {
        return $this->switch_mode;
    }

    public function get_switch_position()
    {
        return $this->switch_position;
    }

    public function get_movie_id()
    {
        return $this->movie_id;
    }

    public function get_torrent_id()
    {
        return $this->torrent_id;
    }

    public function get_torrent_uid()
    {
        return $this->torrent_uid;
    }

    public function get_group_id()
    {
        return $this->group_id;
    }

    public function get_stack()
    {
        return $this->stack;
    }

    public function get_season()
    {
        return $this->season;
    }
}

?>
