<?php

class TorrentHistory
{
    public $dirty = false;

    public $witem_by_uid = null;
    public $witem_by_tag = null;
    public $witem_by_ep = null;

    // EEE TODO: implement some kind of clearing to avoid infinite growth
    public function clear()
    {
        $this->witem_by_uid = null;
        $this->witem_by_tag = null;
        $this->witem_by_ep = null;
    }

    public function make_ep_tag($movie_id, $s, $e)
    {
        if ($s < 0)
            $s = 1;
        return "online_movies:$movie_id:$s:$e";
    }

    public function set_eps($witems)
    {
        $by_eps = HD::get($witems, 'eps', array());

        if (!$this->witem_by_ep)
            $this->witem_by_ep = array();

        foreach ($by_eps as $ep => $whi)
            $this->witem_by_ep[$ep] = $whi;
    }

    public function set($witems)
    {
        $this->dirty = false;

        $this->witem_by_uid = array();
        $this->witem_by_tag = array();
        if (!$witems)
            return;

        $by_tags = HD::get($witems, 'tags', array());
        $by_uids = HD::get($witems, 'uids', array());

        foreach ($by_tags as $tag => $whi)
            $this->witem_by_tag[$tag] = $whi;
        foreach ($by_uids as $uid => $whi)
            $this->witem_by_uid[$uid] = $whi;
    }

    public function get_last_witem($movie_id)
    {
        $witem = null;
        foreach ($this->witem_by_tag as $whi)
        {
            if ($movie_id != HD::get($whi, 'movie_id'))
                continue;
            if (!$witem || $whi->start > $witem->start)
                $witem = $whi;
        }
        return $witem;
    }

    public function make_file_uid($movie_id, $torrent_uid, $file_id)
    {
        return implode(':',
            array("online_movies", $movie_id, $file_id, $torrent_uid));
    }

    public function parse_file_uid($uid) {
        if (!$uid)
            return null;
        $arr = explode(':', $uid, 4);
        if (count($arr) != 4 || $arr[0] != 'online_movies')
            return null;
        return (object) array(
            'movie_id' => $arr[1],
            'file_id' => $arr[2],
            'torrent_uid' => $arr[3]);
    }

    public function make_torrent_tag($torrent_uid) {
        return "online_movies:tuid:$torrent_uid";
    }

    public function parse_sup_data($sup_data) {
        if (!$sup_data)
            return null;

        $res = (object) array();
        $arr = explode(':', $sup_data);
        if (count($arr) >= 1)
            $res->autostart = (int) $arr[0];
        return $res;
    }

    public function get_wh_path()
    {
        return DuneSystem::$properties['tmp_dir_path'] . "/wh.json";
    }

    public function get_eps_wh_path()
    {
        return DuneSystem::$properties['tmp_dir_path'] . "/eps_wh.json";
    }

    public static function update_vip_eitems_by_witem(&$eitems, $witem)
    {
        if (!$witem || !TorrentHistory::with_progress($witem))
            return;

        $with_brackets = (bool) $eitems;

        $pos = $witem->pos > $witem->len ? $witem->len : $witem->pos;

        $percent = TorrentHistory::is_fully_watched($witem) ? 100 :
            intval(($pos * 100 + $witem->len / 2) / $witem->len);

        $prefix = '';
        if ($with_brackets)
        {
            $e = HD::get($witem, 'e', -1);
            if ($e >= 0) {
                $s = HD::get($witem, 's', 1);
                $prefix = "[S$s: E$e";
            } else if (isset($witem->index) && $witem->index >= 0) {
                $prefix = "[#" . ($witem->index + 1);
            }
            if ($prefix && $percent == 100)
                $prefix .= "]";
        }

        $text = "$percent%";
        if ($with_brackets)
        {
            $text = $prefix ? "$text]" : "[$text]";

            $gap = array_shift($eitems);
            if ($gap['type'] == 'gap')
                $gap['width'] = 10;
            array_unshift($eitems, $gap);
        }

        array_unshift($eitems, 
            $percent == 100 ?
            array('type' => 'icon',
                'url' => 'plugin_file://%shell_ext%/icons/watched_small.png',
                'dy' => 2, 'alt' => '100%') :
            array('type' => 'text', 'color' => '8',
                'size' => 'small', 'text' => $text));

        if ($prefix)
        {
            array_unshift($eitems, 
                array('type' => 'gap', 'width' => 10));
            array_unshift($eitems, 
                array('type' => 'text', 'color' => '8',
                    'size' => 'small', 'text' => $prefix));
        }

        array_unshift($eitems, 
            array('type' => 'gap', 'width' => 20));
    }

    public static function with_progress($whi)
    {
        return isset($whi->len) && $whi->len > 0 && $whi->pos >= 0;
    }

    public static function is_fully_watched($whi)
    {
        return self::with_progress($whi) && $whi->pos >= $whi->len - 30;
    }

}

?>
